import Vue from "vue";
import VueRouter from "@/vue-router";

import Vuetify from "@/vuetify";
import "@mdi/font/css/materialdesignicons.css";
import "vuetify/dist/vuetify.min.css";

import { ApolloClient, HttpLink, from } from "@/apollo-boost";
import VueApollo from "@/vue-apollo";
import { InMemoryCache } from "@/apollo-cache-inmemory";
import { onError } from "@/apollo-link-error";

import gql from '@/graphql-tag';

import gqlSnackbarItems from "./snackbarItems.graphql";

import "./css/global.scss";
import VueI18n from "@/vue-i18n";

import messages from "./messages.json";
import dateTimeFormats from "./dateTimeFormats.js";
import routes from "./routes.js";

Vue.use(Vuetify);

const vuetify = new Vuetify({
  icons: {
    iconfont: "mdi", // default - only for display purposes
    values: {
      cancel: "mdi-close-circle-outline",
      delete: "mdi-close-circle-outline",
      success: "mdi-check-circle-outline",
      info: "mdi-information-outline",
      warning: "mdi-alert-outline",
      error: "mdi-alert-octagon-outline",
      prev: "mdi-chevron-left",
      next: "mdi-chevron-right",
      checkboxOn: "mdi-checkbox-marked-outline",
      checkboxIndeterminate: "mdi-minus-box-outline",
      edit: "mdi-pencil-outline",
      preferences: "mdi-cog-outline"
    },
  },
});

Vue.use(VueI18n);

const i18n = new VueI18n({
  locale: "en",
  fallbackLocale: "en",
  messages,
  dateTimeFormats,
});

// Using this function, apps can register their locale files
i18n.registerLocale = function (messages) {
  for (let locale in messages) {
    i18n.mergeLocaleMessage(locale, messages[locale]);
  }
};

Vue.use(VueApollo);
Vue.use(VueRouter);

export const typeDefs = gql`
  type snackbarItem {
    id: ID!
    message: String!
    color: String!
    read: Boolean!
  }

  type Mutation {
    checkSnackbarItem(id: ID!): Boolean
  }
`;

const cache = new InMemoryCache();

const resolvers = {
  Mutation: {
    checkSnackbarItem: (_, {id}, {cache}) => {
      const data = cache.readQuery({query: gqlSnackbarItems});
      const currentItem = data.snackbarItems.find(item => item.id === id);
      currentItem.read = !currentItem.read;
      cache.writeQuery({query: gqlSnackbarItems, data});
      return currentItem.read;
    },
  },
};

function addErrorSnackbarItem (error) {
  let uuid = crypto.randomUUID();
  cache.writeQuery({
    query: gqlSnackbarItems,
    data: {
      snackbarItems: [
        {
          __typename: "snackbarItem",
          id: uuid,
          message: error,
          color: "red",
          read: false
        },
      ]
    },
    variables: {
      id: uuid
    }
  });
};

const errorLink = onError(({ graphQLErrors, networkError }) => {
  if (graphQLErrors)
    graphQLErrors.map(({ message, locations, path }) =>
      addErrorSnackbarItem(`[GraphQL error]: Message: ${message}, Location: ${locations}, Path: ${path}`),
    )

  if (networkError) addErrorSnackbarItem(`[Network error]: ${networkError}`)
});

const httpLink = new HttpLink({
  uri: window.location.origin + "/django/graphql/"
});

const apolloClient = new ApolloClient({
  typeDefs,
  resolvers,
  cache,
  link: from([errorLink, httpLink]),
});

import App from "./App.vue";
import MessageBox from "./components/MessageBox.vue";

Vue.component(MessageBox.name, MessageBox); // Load MessageBox globally as other components depend on it

Vue.use(VueApollo);

const apolloProvider = new VueApollo({
  defaultClient: apolloClient,
});

const router = new VueRouter({
  mode: "history",
  routes,
});

if (document.getElementById("sentry_settings") !== null) {
  const Sentry = import("@sentry/vue");
  const { BrowserTracing } = import("@sentry/tracing");

  const sentry_settings = JSON.parse(
    document.getElementById("sentry_settings").textContent
  );
  Sentry.init({
    Vue,
    dsn: sentry_settings.dsn,
    environment: sentry_settings.environment,
    tracesSampleRate: sentry_settings.traces_sample_rate,
    logError: true,
    integrations: [
      new BrowserTracing({
        routingInstrumentation: Sentry.vueRouterInstrumentation(router),
      }),
    ],
  });
}

const app = new Vue({
  el: "#app",
  apolloProvider,
  vuetify: vuetify,
  render: (h) => h(App),
  data: () => ({
    showCacheAlert: false,
    contentLoading: false,
  }),
  router,
  i18n,
});

window.app = app;
window.router = router;
window.i18n = i18n;

import("./messages.js");
