const fs = require("fs");
const path = require("path");

import { defineConfig, searchForWorkspaceRoot } from "vite";
import vue from "@vitejs/plugin-vue2";
import { nodeResolve } from "@rollup/plugin-node-resolve";
import graphql from "@rollup/plugin-graphql";
import virtual from "@rollup/plugin-virtual";
const license = require("rollup-plugin-license");

const django_values = JSON.parse(fs.readFileSync("./django-vite-values.json"));

function generateAppImporter(appDetails) {
  let code = "let appObjects = {};\n";
  for (const [appPackage, appMeta] of Object.entries(appDetails)) {
    let indexPath = appMeta.assetDir + "/index.js";
    let importAppName =
      appMeta.name.charAt(0).toUpperCase() + appMeta.name.substring(1);

    code += `console.debug("Importing AlekSIS app entrypoint for ${appPackage}");\n`;
    code += `import ${importAppName} from '${indexPath}';\n`;
    code += `appObjects["${appMeta.name}"] = ${importAppName};\n`;
  }
  code += "export default appObjects;\n";
  return code;
}

function generateAppDetails(appDetails) {
  let code = "let messages = {};\n";

  for (const [appPackage, appMeta] of Object.entries(appDetails)) {
    code += `messages["${appMeta.name}"] = JSON.parse('${JSON.stringify(
      appMeta
    )}');\n`;
  }
  code += "export default messages;\n";
  return code;
}

export default defineConfig({
  root: path.resolve(".."),
  base: django_values.static_url,
  build: {
    outDir: path.resolve("./vite_bundles/"),
    manifest: true,
    target: ["firefox102", "chrome108", "safari16"],
    rollupOptions: {
      input: django_values.coreAssetDir + "/index.js",
      output: {
        manualChunks(id) {
          // Split big libraries into own chunks
          if (id.includes("node_modules/vue")) {
            return "vue";
          } else if (id.includes("node_modules/apollo")) {
            return "apollo";
          } else if (id.includes("node_modules/graphql")) {
            return "graphql";
          } else if (id.includes("node_modules/@sentry")) {
            return "sentry";
          } else if (id.includes("node_modules")) {
            // Fallback for all other libraries
            return "vendor";
          }

          // Split each AlekSIS app in its own chunk
          for (const [appPackage, ad] of Object.entries(
            django_values.appDetails
          )) {
            if (id.includes(ad.assetDir + "/index.js")) {
              return appPackage;
            }
          }
        },
      },
    },
  },
  server: {
    strictPort: true,
    origin: "http://127.0.0.1:5173",
    watch: {
      ignored: ["**/*.py", "**/__pycache__/**"],
    },
    fs: {
      allow: [
        searchForWorkspaceRoot(path.resolve("..")),
        ...Object.values(django_values.appDetails).map(
          (details) => details.assetDir
        ),
      ],
    },
  },
  plugins: [
    virtual({
      aleksisAppImporter: generateAppImporter(django_values.appDetails),
      aleksisAppDetails: generateAppDetails(django_values.appDetails),
    }),
    vue(),
    nodeResolve({ modulePaths: [path.resolve("./node_modules")] }),
    graphql(),
    license({
      banner: {
        commentStyle: "ignored",
        content: `Frontend bundle for AlekSIS\nSee ./vendor.LICENSE.txt for copyright information.`,
      },
      thirdParty: {
        allow: {
          test: "MIT OR Apache-2.0 OR 0BSD OR BSD-3-Clause",
          failOnUnlicensed: true,
          failOnViolation: true,
        },
        output: {
          file: path.resolve("./vite_bundles/assets/vendor.LICENSE.txt"),
        },
      },
    }),
  ],
  resolve: {
    alias: {
      "@": path.resolve("./node_modules"),
      "vue": path.resolve("./node_modules/vue/dist/vue.esm.js"),
    },
  },
});
